
;********************************************************************************
;* system.s:
;*	System specific startup and library code for simple IO.
;*
;*	To port GIBL to another 6502 system you may need to change stuff
;*	in this file. It should be fairly obvious, just a few functions in
;*	the first instance; getchar, putchar, newline, getline, and strout.
;*
;*	Then data - data regions would all fit into zero page if RubyOS
;*	allowed it, however RubyOS reserves $90 through $FF for it's own.
;*
;********************************************************************************

;********************************************************************************
;* This file is part of gibl:							*
;*	Gordons Interactive Basic Language                                      *
;********************************************************************************
;*    A Tiny Basic for the 6502 inspired by the NIBL Tiny Basic interpreter;	*
;*    "National Industrial Basic Language" originally for the INS8060 SC/MP	*
;*    system.									*
;*										*
;*    gibl is distributed under a "Source Available" license.			*
;*	It is NOT Open source and must not be treated as such.			*
;*										*
;*    See the file LICENSE for details.						*
;*										*
;*    gibl is Copyright (c) 2023 by Gordon Henderson				*
;********************************************************************************

	.macpack	generic

;********************************************************************************
;* You WILL NEED to change this to include your own operating system
;*	specific hooks for read/write character and so on...
;********************************************************************************


;* Note of the Ruby Memory Map:
;*	It follows the Acorn MOS one relatively closely in that some areas are
;*	reserved - brifly:
;*
;* $0000-$008F	- User ZP storage
;* $0090-$00FF	- Reserved for Ruby OS
;* $0100-$01FF	- 6502 Stack
;* $0200-$02FF	- OS Variables, indirection vectors, etc.
;* $0300-$03FF	- keyboardIn - Used as the getline input buffer.
;* $0400-$07FF	- Reserved for the current "Language"
;* $0800-$0DFF	- Reserved for Ruby OS (input history, other buffers, etc.)
;* $0E00-$7FFF	- Application/Language workspace
;* $8000-$BFFF	- Application/Language code
;* $C000-$FDFF	- RubyOS
;* $FE00-$FEFF	- 65c22 VIA
;* $FF00-$FFFF	- IO area with host MCU and hardware vectors.

	.include	"rubyOs.h"

	.include	"data.h"
	.include	"gibl.h"
	.include	"error.h"

	.include	"system.h"

; Export the keyboardIn variable to the rest of the system.

lineInput	= keyboardIn


;********************************************************************************
;* Code starts here:
;*	This may be ROM. If you alter it, then you will also need to change
;*	the Makefile too.
;********************************************************************************

	.org	$8000


;********************************************************************************
;* Acorn/Ruby MOSify the start...
;*	This MAY not work as-is in a real BBC Micro. It may need work to
;*	make it run correcly as a sideways ROM in a BBC Micro unless you
;*	remove the BASIC ROM first...
;********************************************************************************

rom:	jmp	start					; Language entry
	jmp	start					; Service entry
	.byte	$41					; ROM type (BASIC)
here:	.byte	<copyr-1
	.byte	$1					; Version
	.asciiz	"GIBL"					; Name
copyr:	.byte	"(C) 2023 Gordon Henderson",13,10,0	; Copyright


;********************************************************************************
;* External routines you may need to define/re-define
;*	The code uses the following:
;*		putchar  - Print character in A. (preserve X, Y)
;*		getchar	 - Return character in A. (preserve X, Y)
;*		putstr	 - Output a string in-line with progam code. (preserve X, Y)
;*		getline	 - Read in a line of text
;*		newline  - Output a CR+LF (preserve X, Y)
;*		checkInt - Check for Ctrl-C or whatever.
;********************************************************************************

;********************************************************************************
;* putchar:
;*	Print a single character in A.
;********************************************************************************

putchar:	jmp	osWrch


;********************************************************************************
;* getchar:
;*	Wait-for and read a single character back into A.
;*	Not actually used - Yet ?
;********************************************************************************

getchar:	jmp	osRdch


;********************************************************************************
;* newline:
;*	Output a newline to the terminal
;********************************************************************************

newline:	jmp	osNewl


;********************************************************************************
;* getline:
;*	Read in a line of text.
;*	Here, we just called OSWORD 0.
;*	The RubyOS version handles editing and history recall.
;*	Note the symbol keyboardIn is defined as $0300 in RubyOS.
;*
;*	To remain faithful to the TinyBasic/NIBL one, you need to implement
;*	a bare minimal of editor, remeber it was intended to run on a teletype
;*	in the first instance.
;*
;*	osword 0 will return with Y = to the number of characters typed, so
;*	Y = 0 will be a CR.
;*	Carry is set if the input was terminated by ESC.
;*
;* NOTE: The maximum input line length must be < 127 characters long. Some tests
;*	look for the counter going negative.
;********************************************************************************

.proc	getline
	ldx	#<getLineData
	ldy	#>getLineData
	lda	#0
	jsr	osWord

; Check for some specific OS commands before we hand back to BASIC

; Test for OS command

	lda	lineInput
	cmp	#'*'
	bne	:+
	jsr	osCall
	jmp	getline

:	cmp	#'#'
	bne	:+
	jsr	strout
	.byte	"Reloading",13,10,0

	ldx	#<getGibl
	ldy	#>getGibl
	jsr	osCli
	ldx	#<runGibl
	ldy	#>runGibl
	jmp	osCli

:	rts
	


getLineData:
		.word	keyboardIn		; Address of input buffer
		.byte	76			; Max length
		.byte	32			; Smallest value to accept
		.byte	126			; largest...

getGibl:	.byte	"GET gibl",13
runGibl:	.byte	"/gibl",13

.endproc


;********************************************************************************
;* osCall:
;*	Call an underlying operating system command.
;*	You may just RTS here, or ...
;********************************************************************************

.proc	osCall
	ldx	#<keyboardIn
	ldy	#>keyboardIn
	jsr	osCli
	rts
.endproc

;********************************************************************************
;* strout:
;*	Print a zero-terminated string stored in-line with the program code.
;*	Standard in RubyOS, but a reference implementation is included here.
;*	It assumes we're running in RAM ...
;********************************************************************************

putstr:	jmp	strout


;********************************************************************************
;* checkInt:
;*	Check for a keyboard interrupt. This routine will be called
;*	regularly during program execution and listing
;********************************************************************************

.proc	checkInt
	lda	$FF
	bne	gotInt
	rts

gotInt:
	lda	#0
	sta	$FF
;	lda	#126		; Acknowledge Escape
;	jsr	osByte
	pla			; Remove JSR
	pla
	lda	#eBRK
	jmp	ERR1
.endproc


;********************************************************************************
;* Start here.
;*	Note: RubyOS starts programs in 65C02 emulation mode.
;********************************************************************************

start:
	cld			; Paranoia

; Wipe RAM.
;	We don't need to do this, but lets do it anyway, then we can see what we've
;	written to for debugging later

;	ldx	#0
;	lda	#$FF
;:	sta	fastData,x
;	sta	slowData,x
;	inx
;	cpx	#$90
;	bne	:-

; We don't need much stack, so can use the start of page 1 as the keyboard
;	input buffer if needed

	ldx	#$F0
	txs

; Setup screen and say hello.
;	... possibly a bit too verbose, but RAM is cheap these days.
;	 and my aim is under 4K and we're well on-track there...

	jsr	putstr
	.byte	12,10
	.byte	"  ____ ___ ____  _",13,10
	.byte	" / ___|_ _| __ )| |",13,10
	.byte	"| |  _ | ||  _ \| |",13,10
	.byte	"| |_| || || |_) | |___",13,10
 	.byte	" \____|___|____/|_____|",13,10,10
	.byte	"Gordons Interactive Basic Language",13,10,0


;********************************************************************************
;* Program text start location.
;*	Programs start here and grow up towards the end of RAM.
;*	There is (currently) no check that RAM is exceeded.
;*
;*	Strings start at the end of program text and the built-in
;*	variable TOP will return that address.
;*
;* On non-acorn-ish systems, fix as required. 
;* Get PAGE and store in a known place so we can refer to it later in the code.
;*	We know it's $0E00 in RubyOS, but if someone ports it to a Beeb it may
;*	well be different so use osByte 131.
;********************************************************************************

	lda	#131
	jsr	osByte

	stx	osPageL
	sty	osPageH

; The main system initialisation and run-time.

	jmp	gibl
